@@ -1,5 +1,7 @@
 // Copyright (C) 2024 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "graphs2d/qabstractseries_p.h"
 #include <QtGraphs/qareaseries.h>
@@ -40,22 +42,30 @@ QT_BEGIN_NAMESPACE
     \property QAreaSeries::color
     \brief The fill color of the area.
     The default value is \c Qt::transparent, meaning the color is defined by the theme.
+    \note If \l gradient is set to something other than \c null, these
+    will take precedence over \c color. The \c color will be ignored in this case.
 */
 /*!
     \qmlproperty color AreaSeries::color
     The fill color of the area.
     The default value is \c transparent, meaning the color is defined by the theme.
+    \note If \l gradient is set to something other than \c null, these
+    will take precedence over \c color. The \c color will be ignored in this case.
 */
 
 /*!
     \property QAreaSeries::selectedColor
     \brief The fill color of the area when selected.
     The default value is \c Qt::transparent, meaning the selected color is defined by the theme.
+    \note If \l selectedGradient is set to something other than \c null, these
+    will take precedence over \c selectedColor. The \c selectedColor will be ignored in this case.
 */
 /*!
     \qmlproperty color AreaSeries::selectedColor
     The fill color of the area when selected.
     The default value is \c transparent, meaning the selected color is defined by the theme.
+    \note If \l selectedGradient is set to something other than \c null, these
+    will take precedence over \c selectedColor. The \c selectedColor will be ignored in this case.
 */
 
 /*!
@@ -122,6 +132,56 @@ QT_BEGIN_NAMESPACE
     is considered the lower bound.
 */
 
+/*!
+    \property QAreaSeries::gradient
+    \since 6.11
+    This property defines the fill gradient. By default no gradient is enabled
+    and the value is \c null. In this case the fill will be based on the \l{color} property.
+
+    \note The Gradient type cannot be used here. Rather, prefer using one of
+    the advanced subtypes, like LinearGradient.
+
+    \note If set to something other than \c{null}, the \c gradient will take precedence over
+    \l color.
+*/
+/*!
+    \qmlproperty ShapeGradient AreaSeries::gradient
+    \since 6.11
+    This property defines the fill gradient. By default no gradient is enabled
+    and the value is \c null. In this case the fill will be based on the \l{color} property.
+
+    \note The Gradient type cannot be used here. Rather, prefer using one of
+    the advanced subtypes, like LinearGradient.
+
+    \note If set to something other than \c{null}, the \c gradient will take precedence over
+    \l color.
+*/
+
+/*!
+    \property QAreaSeries::selectedGradient
+    \since 6.11
+    This property defines the fill gradient of the area when selected. By default no gradient is enabled
+    and the value is \c null. In this case the fill will be based on the \l{selectedColor} property.
+
+    \note The Gradient type cannot be used here. Rather, prefer using one of
+    the advanced subtypes, like LinearGradient.
+
+    \note If set to something other than \c{null}, the \c selectedGradient will take precedence over
+    \l selectedColor.
+*/
+/*!
+    \qmlproperty ShapeGradient AreaSeries::selectedGradient
+    \since 6.11
+    This property defines the fill gradient of the area when selected. By default no gradient is enabled
+    and the value is \c null. In this case the fill will be based on the \l{selectedColor} property.
+
+    \note The Gradient type cannot be used here. Rather, prefer using one of
+    the advanced subtypes, like LinearGradient.
+
+    \note If set to something other than \c{null}, the \c selectedGradient will take precedence over
+    \l selectedColor.
+*/
+
 /*!
     \qmlsignal AreaSeries::colorChanged(color newColor)
     This signal is emitted when the area \l color changes to \a newColor.
@@ -165,6 +225,18 @@ QT_BEGIN_NAMESPACE
     This signal is emitted when the lower series changes.
 */
 
+/*!
+    \qmlsignal AreaSeries::gradientChanged();
+    \since 6.11
+    This signal is emitted when the gradient changes.
+*/
+
+/*!
+    \qmlsignal AreaSeries::selectedGradientChanged();
+    \since 6.11
+    This signal is emitted when the gradient of a selected area changes.
+*/
+
 /*!
     \qmlsignal AreaSeries::clicked(point point)
     This signal is emitted when the user clicks or taps an area graph.
@@ -222,6 +294,7 @@ void QAreaSeries::setColor(QColor newColor)
     }
     d->m_color = newColor;
     emit colorChanged(newColor);
+    emit update();
 }
 
 QColor QAreaSeries::selectedColor() const
@@ -241,6 +314,7 @@ void QAreaSeries::setSelectedColor(QColor newSelectedColor)
     }
     d->m_selectedColor = newSelectedColor;
     emit selectedColorChanged(newSelectedColor);
+    emit update();
 }
 
 QColor QAreaSeries::borderColor() const
@@ -260,6 +334,7 @@ void QAreaSeries::setBorderColor(QColor newBorderColor)
     }
     d->m_borderColor = newBorderColor;
     emit borderColorChanged(newBorderColor);
+    emit update();
 }
 
 QColor QAreaSeries::selectedBorderColor() const
@@ -279,6 +354,7 @@ void QAreaSeries::setSelectedBorderColor(QColor newSelectedBorderColor)
     }
     d->m_selectedBorderColor = newSelectedBorderColor;
     emit selectedBorderColorChanged(newSelectedBorderColor);
+    emit update();
 }
 
 qreal QAreaSeries::borderWidth() const
@@ -290,13 +366,14 @@ qreal QAreaSeries::borderWidth() const
 void QAreaSeries::setBorderWidth(qreal newBorderWidth)
 {
     Q_D(QAreaSeries);
-    if (qFuzzyCompare(d->m_borderWidth, newBorderWidth)) {
+    if (QtPrivate::fuzzyCompare(d->m_borderWidth, newBorderWidth)) {
         qCDebug(lcProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newBorderWidth);
         return;
     }
     d->m_borderWidth = newBorderWidth;
     emit borderWidthChanged();
+    emit update();
 }
 
 bool QAreaSeries::isSelected() const
@@ -340,6 +417,7 @@ void QAreaSeries::setUpperSeries(QXYSeries *newUpperSeries)
     connect(newUpperSeries, &QXYSeries::update, this, &QAreaSeries::update);
 
     emit upperSeriesChanged();
+    emit update();
 }
 
 QXYSeries *QAreaSeries::lowerSeries() const
@@ -365,6 +443,45 @@ void QAreaSeries::setLowerSeries(QXYSeries *newLowerSeries)
     connect(newLowerSeries, &QXYSeries::update, this, &QAreaSeries::update);
 
     emit lowerSeriesChanged();
+    emit update();
+}
+
+QQuickShapeGradient* QAreaSeries::gradient() const
+{
+    Q_D(const QAreaSeries);
+    return d->m_gradient;
+}
+
+void QAreaSeries::setGradient(QQuickShapeGradient* newGradient)
+{
+    Q_D(QAreaSeries);
+    if (d->m_gradient == newGradient) {
+        qCDebug(lcProperties2D) << __FUNCTION__
+            << "value is already set to:" << newGradient;
+        return;
+    }
+    d->m_gradient = newGradient;
+    emit gradientChanged(newGradient);
+    emit update();
+}
+
+QQuickShapeGradient* QAreaSeries::selectedGradient() const
+{
+    Q_D(const QAreaSeries);
+    return d->m_selectedGradient;
+}
+
+void QAreaSeries::setSelectedGradient(QQuickShapeGradient* newSelectedGradient)
+{
+    Q_D(QAreaSeries);
+    if (d->m_selectedGradient == newSelectedGradient) {
+        qCDebug(lcProperties2D) << __FUNCTION__
+            << "value is already set to:" << newSelectedGradient;
+        return;
+    }
+    d->m_selectedGradient = newSelectedGradient;
+    emit selectedGradientChanged(newSelectedGradient);
+    emit update();
 }
 
 QAreaSeriesPrivate::QAreaSeriesPrivate()
@@ -372,3 +489,5 @@ QAreaSeriesPrivate::QAreaSeriesPrivate()
 {}
 
 QT_END_NAMESPACE
+
+#include "moc_qareaseries.cpp"
