@@ -1,5 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "graphs2d/qabstractseries.h"
 #include "graphs2d/qabstractseries_p.h"
@@ -27,6 +29,10 @@
 #ifdef USE_POINTS
 #include <private/pointrenderer_p.h>
 #endif
+#ifdef USE_CUSTOMGRAPH
+#include <QtGraphs/qcustomseries.h>
+#include <private/customrenderer_p.h>
+#endif
 #include <QTimer>
 #include <QtQuick/private/qquickpinchhandler_p.h>
 #include <QtQuick/private/qquickrectangle_p.h>
@@ -68,6 +74,12 @@ Q_TRACE_POINT(qtgraphs, QGraphs2DGraphsViewCreatePieRenderer_exit);
 Q_TRACE_POINT(qtgraphs, QGraphs2DGraphsViewCreateAreaRenderer_entry);
 Q_TRACE_POINT(qtgraphs, QGraphs2DGraphsViewCreateAreaRenderer_exit);
 
+/*!
+    \class QGraphsView
+    \inmodule QtGraphs
+    \internal
+*/
+
 /*!
     \qmltype GraphsView
     \nativetype QGraphsView
@@ -138,7 +150,7 @@ void QGraphsView::onPinchGrabChanged(QPointingDevice::GrabTransition transition,
 }
 
 /*!
-    \qmlmethod GraphsView::addSeries(AbstractSeries series)
+    \qmlmethod void GraphsView::addSeries(AbstractSeries series)
     Appends a \a series into GraphsView.
     If the \a series is null, it will not be added. If the \a series already
     belongs to the graph, it will be moved into the end.
@@ -154,7 +166,7 @@ void QGraphsView::addSeries(QObject *series)
 }
 
 /*!
-    \qmlmethod GraphsView::insertSeries(int index, AbstractSeries series)
+    \qmlmethod void GraphsView::insertSeries(int index, AbstractSeries series)
     Inserts a \a series at the position specified by \a index.
     If the \a series is null, it will not be inserted. If the \a series already
     belongs to the graph, it will be moved into \a index.
@@ -183,6 +195,10 @@ void QGraphsView::insertSeries(qsizetype index, QObject *object)
         } else {
             m_seriesList.insert(index, series);
 
+            QObject::connect(series,
+                             &QAbstractSeries::update,
+                             this,
+                             &QGraphsView::updateComponentSizes);
             QObject::connect(series, &QAbstractSeries::update,
                              this, &QGraphsView::polishAndUpdate);
             QObject::connect(series, &QAbstractSeries::hoverEnter,
@@ -206,7 +222,7 @@ void QGraphsView::insertSeries(qsizetype index, QObject *object)
 }
 
 /*!
-    \qmlmethod GraphsView::removeSeries(AbstractSeries series)
+    \qmlmethod void GraphsView::removeSeries(AbstractSeries series)
     Removes the \a series from the graph.
 */
 /*!
@@ -231,7 +247,7 @@ void QGraphsView::removeSeries(QObject *object)
 }
 
 /*!
-    \qmlmethod GraphsView::removeSeries(int index)
+    \qmlmethod void GraphsView::removeSeries(int index)
     Removes the series specified by \a index from the graph.
 */
 /*!
@@ -277,22 +293,19 @@ void QGraphsView::addAxis(QAbstractAxis *axis)
         createAxisRenderer();
         polishAndUpdate();
         QObject::connect(axis, &QAbstractAxis::update, this, &QGraphsView::polishAndUpdate);
-        QObject::connect(axis,
-                         &QAbstractAxis::visibleChanged,
-                         this,
-                         &QGraphsView::updateComponentSizes);
+        QObject::connect(axis, &QAbstractAxis::update, this, &QGraphsView::updateComponentSizes);
     }
 }
 
 void QGraphsView::removeAxis(QAbstractAxis *axis)
 {
-    if (m_axisX == axis || m_axisY == axis) {
+    if (axis) {
         axis->d_func()->setGraph(nullptr);
         QObject::disconnect(axis, &QAbstractAxis::update, this, &QGraphsView::polishAndUpdate);
         QObject::disconnect(axis,
-                            &QAbstractAxis::visibleChanged,
+                            &QAbstractAxis::update,
                             this,
-                            &QGraphsView::updateComponentSizes);
+                             &QGraphsView::updateComponentSizes);
     }
 
     if (m_axisX == axis)
@@ -374,6 +387,17 @@ void QGraphsView::createAreaRenderer()
 }
 #endif
 
+#ifdef USE_CUSTOMGRAPH
+void QGraphsView::createCustomRenderer()
+{
+    if (!m_customRenderer) {
+        qCDebug(lcGraphs2D, "creating custom renderer.");
+        m_customRenderer = new CustomRenderer(this, clipPlotArea());
+        updateComponentSizes();
+    }
+}
+#endif
+
 /*!
     \property QGraphsView::axisXSmoothing
     \brief Controls the graph X axis smoothing (antialiasing) amount.
@@ -391,7 +415,7 @@ qreal QGraphsView::axisXSmoothing() const
 
 void QGraphsView::setAxisXSmoothing(qreal smoothing)
 {
-    if (qFuzzyCompare(m_axisXSmoothing, smoothing)) {
+    if (QtPrivate::fuzzyCompare(m_axisXSmoothing, smoothing)) {
         qCDebug(lcViewProperties2D, "%s axis smoothing is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)),
                 smoothing);
@@ -419,7 +443,7 @@ qreal QGraphsView::axisYSmoothing() const
 
 void QGraphsView::setAxisYSmoothing(qreal smoothing)
 {
-    if (qFuzzyCompare(m_axisYSmoothing, smoothing)) {
+    if (QtPrivate::fuzzyCompare(m_axisYSmoothing, smoothing)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), smoothing);
         return;
@@ -446,7 +470,7 @@ qreal QGraphsView::gridSmoothing() const
 
 void QGraphsView::setGridSmoothing(qreal smoothing)
 {
-    if (qFuzzyCompare(m_gridSmoothing, smoothing)) {
+    if (QtPrivate::fuzzyCompare(m_gridSmoothing, smoothing)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), smoothing);
         return;
@@ -527,7 +551,7 @@ qreal QGraphsView::shadowBarWidth() const
 
 void QGraphsView::setShadowBarWidth(qreal newShadowBarWidth)
 {
-    if (qFuzzyCompare(m_shadowBarWidth, newShadowBarWidth)) {
+    if (QtPrivate::fuzzyCompare(m_shadowBarWidth, newShadowBarWidth)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newShadowBarWidth);
         return;
@@ -554,7 +578,7 @@ qreal QGraphsView::shadowXOffset() const
 
 void QGraphsView::setShadowXOffset(qreal newShadowXOffset)
 {
-    if (qFuzzyCompare(m_shadowXOffset, newShadowXOffset)) {
+    if (QtPrivate::fuzzyCompare(m_shadowXOffset, newShadowXOffset)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newShadowXOffset);
         return;
@@ -581,7 +605,7 @@ qreal QGraphsView::shadowYOffset() const
 
 void QGraphsView::setShadowYOffset(qreal newShadowYOffset)
 {
-    if (qFuzzyCompare(m_shadowYOffset, newShadowYOffset)) {
+    if (QtPrivate::fuzzyCompare(m_shadowYOffset, newShadowYOffset)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newShadowYOffset);
         return;
@@ -608,7 +632,7 @@ qreal QGraphsView::shadowSmoothing() const
 
 void QGraphsView::setShadowSmoothing(qreal smoothing)
 {
-    if (qFuzzyCompare(m_shadowSmoothing, smoothing)) {
+    if (QtPrivate::fuzzyCompare(m_shadowSmoothing, smoothing)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), smoothing);
         return;
@@ -693,6 +717,15 @@ void QGraphsView::updateComponentSizes()
 
     }
 #endif
+#ifdef USE_CUSTOMGRAPH
+    if (m_customRenderer) {
+        m_customRenderer->setX(m_plotArea.x());
+        m_customRenderer->setY(m_plotArea.y());
+        m_customRenderer->setSize(m_plotArea.size());
+        qCDebug(lcEvents2D) << "custom graph size:" << m_plotArea.size();
+        qCDebug(lcEvents2D, "customaRenderer plotArea x: %f y: %f", m_plotArea.x(), m_plotArea.y());
+    }
+#endif
 }
 
 void QGraphsView::componentComplete()
@@ -848,6 +881,13 @@ QSGNode *QGraphsView::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaintN
                 m_areaRenderer->updateSeries(areaSeries);
         }
 #endif
+
+#ifdef USE_CUSTOMGRAPH
+        if (m_customRenderer) {
+            if (auto customSeries = qobject_cast<QCustomSeries *>(series))
+                m_customRenderer->updateSeries(customSeries);
+        }
+#endif
     }
 
 #ifdef USE_BARGRAPH
@@ -881,6 +921,14 @@ QSGNode *QGraphsView::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaintN
     }
 #endif
 
+#ifdef USE_CUSTOMGRAPH
+    if (m_customRenderer) {
+        auto &cleanupSeriesList = m_cleanupSeriesList[4];
+        m_customRenderer->afterUpdate(cleanupSeriesList);
+        cleanupSeriesList.clear();
+    }
+#endif
+
     // Now possibly dirty theme has been taken into use
     m_theme->resetThemeDirty();
 
@@ -937,15 +985,23 @@ void QGraphsView::updatePolish()
     float highestPointZ = -std::numeric_limits<float>::max();
     float highestPieZ = -std::numeric_limits<float>::max();
     float highestAreaZ = -std::numeric_limits<float>::max();
+    float highestCustomZ = -std::numeric_limits<float>::max();
 
     // Polish for all series
+#ifdef USE_BARGRAPH
+    int barSeriesIndex = 0;
+    int barSeriesCount =
+            std::count_if(m_seriesList.begin(), m_seriesList.end(),
+                          [](const auto &series) { return qobject_cast<QBarSeries *>(series); });
+#endif
     for (auto series : std::as_const(m_seriesList)) {
 #ifdef USE_BARGRAPH
         if (m_barsRenderer) {
             if (auto barSeries = qobject_cast<QBarSeries *>(series)) {
-                m_barsRenderer->handlePolish(barSeries);
+                m_barsRenderer->handlePolish(barSeries, barSeriesIndex, barSeriesCount);
                 if (barSeries->zValue() > highestBarsZ)
                     highestBarsZ = barSeries->zValue();
+                barSeriesIndex++;
             }
         }
 #endif
@@ -997,6 +1053,16 @@ void QGraphsView::updatePolish()
             }
         }
 #endif
+
+#ifdef USE_CUSTOMGRAPH
+        if (m_customRenderer) {
+            if (auto customSeries = qobject_cast<QCustomSeries *>(series)) {
+                m_customRenderer->handlePolish(customSeries);
+                if (customSeries->zValue() > highestCustomZ)
+                    highestCustomZ = customSeries->zValue();
+            }
+        }
+#endif
     }
 
 #ifdef USE_BARGRAPH
@@ -1031,6 +1097,14 @@ void QGraphsView::updatePolish()
             m_pieRenderer->setZ(highestPieZ);
     }
 #endif
+#ifdef USE_CUSTOMGRAPH
+    if (m_customRenderer) {
+        auto &cleanupSeriesList = m_cleanupSeriesList[4];
+        m_customRenderer->afterPolish(cleanupSeriesList);
+        if (highestCustomZ > -std::numeric_limits<float>::max())
+            m_customRenderer->setZ(highestCustomZ);
+    }
+#endif
 }
 
 void QGraphsView::polishAndUpdate()
@@ -1130,7 +1204,7 @@ qreal QGraphsView::marginTop() const
 
 void QGraphsView::setMarginTop(qreal newMarginTop)
 {
-    if (qFuzzyCompare(m_marginTop, newMarginTop)) {
+    if (QtPrivate::fuzzyCompare(m_marginTop, newMarginTop)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newMarginTop);
         return;
@@ -1153,7 +1227,7 @@ qreal QGraphsView::marginBottom() const
 
 void QGraphsView::setMarginBottom(qreal newMarginBottom)
 {
-    if (qFuzzyCompare(m_marginBottom, newMarginBottom)) {
+    if (QtPrivate::fuzzyCompare(m_marginBottom, newMarginBottom)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newMarginBottom);
         return;
@@ -1176,7 +1250,7 @@ qreal QGraphsView::marginLeft() const
 
 void QGraphsView::setMarginLeft(qreal newMarginLeft)
 {
-    if (qFuzzyCompare(m_marginLeft, newMarginLeft)) {
+    if (QtPrivate::fuzzyCompare(m_marginLeft, newMarginLeft)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newMarginLeft);
         return;
@@ -1199,7 +1273,7 @@ qreal QGraphsView::marginRight() const
 
 void QGraphsView::setMarginRight(qreal newMarginRight)
 {
-    if (qFuzzyCompare(m_marginRight, newMarginRight)) {
+    if (QtPrivate::fuzzyCompare(m_marginRight, newMarginRight)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newMarginRight);
         return;
@@ -1257,6 +1331,10 @@ void QGraphsView::setClipPlotArea(bool enabled)
     if (m_barsRenderer)
         m_barsRenderer->setClip(m_clipPlotArea);
 #endif
+#ifdef USE_CUSTOMGRAPH
+    if (m_customRenderer)
+        m_customRenderer->setClip(m_clipPlotArea);
+#endif
 }
 
 /*!
@@ -1312,14 +1390,26 @@ void QGraphsView::updateAxisAreas()
     int yCount = 0;
     int topCount = 0;
     int leftCount = 0;
+    int xTitleCount = 0;
+    int yTitleCount = 0;
+    int topTitleCount = 0;
+    int leftTitleCount = 0;
 
-    calculateAxisCounts(&xCount, &yCount, &leftCount, &topCount);
+    calculateAxisCounts(&xCount,
+                        &yCount,
+                        &leftCount,
+                        &topCount,
+                        &xTitleCount,
+                        &yTitleCount,
+                        &leftTitleCount,
+                        &topTitleCount);
 
-    m_x1AxisArea = { r.x(),
-                     r.y() + r.height() + m_axisHeight * (2 - xCount)
-                        - m_axisHeight * (2 - topCount),
-                     r.width() - m_axisWidth * yCount,
-                     m_axisHeight };
+    m_x1AxisArea = {r.x(),
+                    r.y() + r.height() + m_axisHeight * (2 - xCount)
+                        + m_axisTitleMargin * (2 - xTitleCount) - m_axisHeight * (2 - topCount)
+                        - m_axisTitleMargin * (2 - topTitleCount),
+                    r.width() - m_axisWidth * yCount - m_axisTitleMargin * yTitleCount,
+                    m_axisHeight};
     m_x1AxisLabelsArea = { m_x1AxisArea.x(),
                          m_x1AxisArea.y() + m_axisTickersHeight + m_axisXLabelsMargin,
                          m_x1AxisArea.width(),
@@ -1329,7 +1419,10 @@ void QGraphsView::updateAxisAreas()
                           m_x1AxisArea.width(),
                           m_axisTickersHeight };
 
-    m_x2AxisArea = { r.x(), r.y(), r.width() - m_axisWidth * yCount, m_axisHeight };
+    m_x2AxisArea = {r.x(),
+                    r.y(),
+                    r.width() - m_axisWidth * yCount - m_axisTitleMargin * yTitleCount,
+                    m_axisHeight};
     m_x2AxisLabelsArea = { m_x2AxisArea.x(),
                           m_x2AxisArea.y(),
                           m_x2AxisArea.width(),
@@ -1339,7 +1432,10 @@ void QGraphsView::updateAxisAreas()
                            m_x2AxisArea.width(),
                            m_axisTickersHeight };
 
-    m_y1AxisArea = { r.x(), r.y(), m_axisWidth, r.height() - m_axisHeight * xCount };
+    m_y1AxisArea = {r.x(),
+                    r.y(),
+                    m_axisWidth,
+                    r.height() - m_axisHeight * xCount - m_axisTitleMargin * xTitleCount};
     m_y1AxisLabelsArea = { m_y1AxisArea.x(),
                           m_y1AxisArea.y(),
                           m_axisLabelsWidth,
@@ -1349,11 +1445,12 @@ void QGraphsView::updateAxisAreas()
                            m_axisTickersWidth,
                            m_y1AxisArea.height() };
 
-    m_y2AxisArea = { r.x() + r.width() + m_axisWidth * (2 - yCount)
-                        - m_axisWidth * (2 - leftCount),
-                     r.y(),
-                     m_axisWidth,
-                     r.height() - m_axisHeight * xCount };
+    m_y2AxisArea = {r.x() + r.width() + m_axisWidth * (2 - yCount)
+                        + m_axisTitleMargin * (2 - yTitleCount) - m_axisWidth * (2 - leftCount)
+                        - m_axisTitleMargin * (2 - leftTitleCount),
+                    r.y(),
+                    m_axisWidth,
+                    r.height() - m_axisHeight * xCount - m_axisTitleMargin * xTitleCount};
     m_y2AxisLabelsArea = { m_y2AxisArea.x() + m_axisTickersWidth + m_axisYLabelsMargin,
                           m_y2AxisArea.y(),
                           m_axisLabelsWidth,
@@ -1376,13 +1473,24 @@ void QGraphsView::updatePlotArea()
     int yCount = 0;
     int topCount = 0;
     int leftCount = 0;
+    int xTitleCount = 0;
+    int yTitleCount = 0;
+    int topTitleCount = 0;
+    int leftTitleCount = 0;
 
-    calculateAxisCounts(&xCount, &yCount, &leftCount, &topCount);
+    calculateAxisCounts(&xCount,
+                        &yCount,
+                        &leftCount,
+                        &topCount,
+                        &xTitleCount,
+                        &yTitleCount,
+                        &leftTitleCount,
+                        &topTitleCount);
 
-    y += m_axisHeight * topCount;
-    x += m_axisWidth * leftCount;
-    h -= m_axisHeight * xCount;
-    w -= m_axisWidth * yCount;
+    y += m_axisHeight * topCount + m_axisTitleMargin * topTitleCount;
+    x += m_axisWidth * leftCount + m_axisTitleMargin * leftTitleCount;
+    h -= m_axisHeight * xCount + m_axisTitleMargin * xTitleCount;
+    w -= m_axisWidth * yCount + m_axisTitleMargin * yTitleCount;
 
     w = qMax(w, 0.0);
     h = qMax(h, 0.0);
@@ -1514,6 +1622,25 @@ void QGraphsView::setOrientation(Qt::Orientation newOrientation)
     polishAndUpdate();
 }
 
+/*
+    mapX and mapY functions are used to map custom series data values
+    to the real pixel values of the GraphsView
+*/
+qreal QGraphsView::mapX(QCustomSeries *series, qreal x)
+{
+    return m_customRenderer->mapX(m_axisRenderer, series, x);
+}
+
+qreal QGraphsView::mapY(QCustomSeries *series, qreal y)
+{
+    return m_customRenderer->mapY(m_axisRenderer, series, y);
+}
+
+CustomRenderer *QGraphsView::customRenderer() const
+{
+    return m_customRenderer;
+}
+
 /*!
     \enum QGraphsView::ZoomStyle
     This enum value describes the zoom style of the graph:
@@ -1693,7 +1820,7 @@ qreal QGraphsView::zoomSensitivity() const
 
 void QGraphsView::setZoomSensitivity(qreal newZoomSensitivity)
 {
-    if (qFuzzyCompare(m_zoomSensitivity, newZoomSensitivity)) {
+    if (QtPrivate::fuzzyCompare(m_zoomSensitivity, newZoomSensitivity)) {
         qCDebug(lcViewProperties2D, "%s value is already set to: %.1f",
                 qUtf8Printable(QLatin1String(__FUNCTION__)), newZoomSensitivity);
         return;
@@ -1702,36 +1829,57 @@ void QGraphsView::setZoomSensitivity(qreal newZoomSensitivity)
     emit zoomSensitivityChanged();
 }
 
-void QGraphsView::calculateAxisCounts(int *xCount, int *yCount, int *leftCount, int *topCount)
+void QGraphsView::calculateAxisCounts(int *xCount, int *yCount, int *leftCount, int *topCount,
+                                      int *xTitleCount, int *yTitleCount, int *leftTitleCount, int *topTitleCount)
 {
     if (axisY()) {
         (*yCount)++;
+        if (axisY()->isTitleVisible() && !axisY()->titleText().isEmpty())
+            (*yTitleCount)++;
 
-        if (axisY()->alignment() == Qt::AlignLeft)
+        if (axisY()->alignment() == Qt::AlignLeft) {
             (*leftCount)++;
+            if (axisY()->isTitleVisible() && !axisY()->titleText().isEmpty())
+                (*leftTitleCount)++;
+        }
     }
 
     if (axisX()) {
         (*xCount)++;
+        if (axisX()->isTitleVisible() && !axisX()->titleText().isEmpty())
+            (*xTitleCount)++;
 
-        if (axisX()->alignment() == Qt::AlignTop)
+        if (axisX()->alignment() == Qt::AlignTop) {
             (*topCount)++;
+            if (axisX()->isTitleVisible() && !axisX()->titleText().isEmpty())
+                (*topTitleCount)++;
+        }
     }
 
     for (auto&& s : m_seriesList) {
         if (auto series = qobject_cast<QAbstractSeries *>(s)) {
             if (series->axisY() && series->axisY() != axisY()) {
                 (*yCount)++;
+                if (series->axisY()->isTitleVisible() && !series->axisY()->titleText().isEmpty())
+                    (*yTitleCount)++;
 
-                if (series->axisY()->alignment() == Qt::AlignLeft)
+                if (series->axisY()->alignment() == Qt::AlignLeft) {
                     (*leftCount)++;
+                    if (series->axisY()->isTitleVisible() && !series->axisY()->titleText().isEmpty())
+                        (*leftTitleCount)++;
+                }
             }
 
             if (series->axisX() && series->axisX() != axisX()) {
                 (*xCount)++;
+                if (series->axisX()->isTitleVisible() && !series->axisX()->titleText().isEmpty())
+                    (*xTitleCount)++;
 
-                if (series->axisX()->alignment() == Qt::AlignTop)
+                if (series->axisX()->alignment() == Qt::AlignTop) {
                     (*topCount)++;
+                    if (series->axisX()->isTitleVisible() && !series->axisX()->titleText().isEmpty())
+                        (*topTitleCount)++;
+                }
             }
         }
     }
@@ -1739,7 +1887,27 @@ void QGraphsView::calculateAxisCounts(int *xCount, int *yCount, int *leftCount,
     if (m_orientation == Qt::Horizontal) {
         qSwap((*xCount), (*yCount));
         qSwap((*leftCount), (*topCount));
+        qSwap((*xTitleCount), (*yTitleCount));
+        qSwap((*leftTitleCount), (*topTitleCount));
     }
+
+    if ((*xTitleCount) > 0) {
+        (*xTitleCount)--;
+        if ((*xTitleCount) > 0)
+            (*xTitleCount)--;
+    }
+
+    if ((*yTitleCount) > 0) {
+        (*yTitleCount)--;
+        if ((*yTitleCount) > 0)
+            (*yTitleCount)--;
+    }
+
+    if ((*leftTitleCount) > 0)
+        (*leftTitleCount)--;
+
+    if ((*topTitleCount) > 0)
+        (*topTitleCount)--;
 }
 
 int QGraphsView::getSeriesRendererIndex(QAbstractSeries *series)
@@ -1761,9 +1929,14 @@ int QGraphsView::getSeriesRendererIndex(QAbstractSeries *series)
         case QAbstractSeries::SeriesType::Pie:
             index = 3;
             break;
+        case QAbstractSeries::SeriesType::Custom:
+            index = 4;
+            break;
         }
     }
     return index;
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qgraphsview_p.cpp"
