@@ -1,5 +1,7 @@
 // Copyright (C) 2024 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include <QtGraphs/qareaseries.h>
 #include <QtGraphs/qsplineseries.h>
@@ -162,10 +164,14 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
             ? series->borderColor()
             : borderColors.at(index);
 
+    QQuickShapeGradient *gradient = series->gradient();
+
     if (series->isSelected()) {
         color = series->selectedColor().alpha() != 0 ? series->selectedColor() : color.lighter();
         borderColor = series->selectedBorderColor().alpha() != 0 ? series->selectedBorderColor()
                                                                  : borderColor.lighter();
+        if (series->selectedGradient())
+            gradient = series->selectedGradient();
     }
 
     qreal borderWidth = series->borderWidth();
@@ -175,6 +181,7 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
     group->shapePath->setStrokeWidth(borderWidth);
     group->shapePath->setStrokeColor(borderColor);
     group->shapePath->setFillColor(color);
+    group->shapePath->setFillGradient(gradient);
     group->shapePath->setCapStyle(QQuickShapePath::CapStyle::SquareCap);
 
     auto &&upperPoints = upper->points();
@@ -535,3 +542,5 @@ void AreaRenderer::onPressedChanged()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_arearenderer_p.cpp"
