@@ -1,5 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #ifdef USE_LINEGRAPH
 #include <QtGraphs/qlineseries.h>
@@ -17,6 +19,7 @@
 #include <private/qabstractseries_p.h>
 #include <private/qgraphsview_p.h>
 #include <private/qxyseries_p.h>
+#include <private/charthelpers_p.h>
 
 #include <qtgraphs_tracepoints_p.h>
 
@@ -219,7 +222,7 @@ void PointRenderer::updatePointDelegate(
 
     marker->setX(x - marker->width() / 2.0);
     marker->setY(y - marker->height() / 2.0);
-    marker->setVisible(true);
+    marker->setVisible(isValidValue(x) && isValidValue(y));
 
     rect = QRectF(x - marker->width() / 2.0,
                   y - marker->height() / 2.0,
@@ -381,9 +384,23 @@ void PointRenderer::updateLineSeries(QLineSeries *series, QLegendData &legendDat
     else if (capStyle == Qt::PenCapStyle::RoundCap)
         group->shapePath->setCapStyle(QQuickShapePath::CapStyle::RoundCap);
 
+    Qt::PenJoinStyle joinStyle = series->joinStyle();
+    if (joinStyle == Qt::PenJoinStyle::BevelJoin)
+        group->shapePath->setJoinStyle(QQuickShapePath::JoinStyle::BevelJoin);
+    else if (joinStyle == Qt::PenJoinStyle::MiterJoin)
+        group->shapePath->setJoinStyle(QQuickShapePath::JoinStyle::MiterJoin);
+    else if (joinStyle == Qt::PenJoinStyle::RoundJoin)
+        group->shapePath->setJoinStyle(QQuickShapePath::JoinStyle::RoundJoin);
+
+    group->shapePath->setStrokeStyle(QQuickShapePath::StrokeStyle(series->strokeStyle()));
+    group->shapePath->setDashOffset(series->dashOffset());
+    group->shapePath->setDashPattern(series->dashPattern());
+
     auto &painterPath = group->painterPath;
     painterPath.clear();
 
+    QPointF previousRenderCoordinates;
+
     Q_TRACE(QGraphs2DPointRendererUpdateLineSeries_entry);
     if (series->isVisible()) {
         auto &&points = series->points();
@@ -399,8 +416,33 @@ void PointRenderer::updateLineSeries(QLineSeries *series, QLegendData &legendDat
             y *= series->valuesMultiplier();
             if (i == 0)
                 painterPath.moveTo(x, y);
-            else
-                painterPath.lineTo(x, y);
+            else {
+                if (!isValidValue(previousRenderCoordinates.x()) || !isValidValue(previousRenderCoordinates.y())) {
+                    if (isValidValue(x) && isValidValue(y))
+                        painterPath.moveTo(x, y);
+                } else if (isValidValue(x) && isValidValue(y)) {
+                    switch (series->lineStyle()) {
+                    case QLineSeries::LineStyle::StepLeft:
+                        painterPath.lineTo(x, previousRenderCoordinates.y());
+                        painterPath.lineTo(x, y);
+                        break;
+                    case QLineSeries::LineStyle::StepRight:
+                        painterPath.lineTo(previousRenderCoordinates.x(), y);
+                        painterPath.lineTo(x, y);
+                        break;
+                    case QLineSeries::LineStyle::StepCenter: {
+                        qreal half = (x + previousRenderCoordinates.x()) / 2;
+                        painterPath.lineTo(half, previousRenderCoordinates.y());
+                        painterPath.lineTo(half, y);
+                        painterPath.lineTo(x, y);
+                    } break;
+                    default:
+                        painterPath.lineTo(x, y);
+                        break;
+                    }
+                }
+            }
+            previousRenderCoordinates = {x, y};
 
             if (group->currentMarker) {
                 updatePointDelegate(series, group, i, x, y);
@@ -948,3 +990,5 @@ QPointF PointRenderer::reverseRenderCoordinates(QAbstractSeries *series, qreal x
 }
 
 QT_END_NAMESPACE
+
+#include "moc_pointrenderer_p.cpp"
