@@ -1,5 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "graphs2d/xychart/qxyseries_p.h"
 #include <QtGraphs/qscatterseries.h>
@@ -146,3 +148,5 @@ QScatterSeriesPrivate::QScatterSeriesPrivate()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qscatterseries.cpp"
