@@ -1,5 +1,7 @@
 // Copyright (C) 2024 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #ifndef QTGRAPHS_QSPLINESERIES_H
 #define QTGRAPHS_QSPLINESERIES_H
@@ -16,6 +18,8 @@ class Q_GRAPHS_EXPORT QSplineSeries : public QXYSeries
     Q_OBJECT
     Q_PROPERTY(qreal width READ width WRITE setWidth NOTIFY widthChanged FINAL)
     Q_PROPERTY(Qt::PenCapStyle capStyle READ capStyle WRITE setCapStyle NOTIFY capStyleChanged FINAL)
+    Q_PROPERTY(
+        bool optimized READ isOptimized WRITE setOptimized NOTIFY optimizedChanged REVISION(6, 11))
 
     QML_NAMED_ELEMENT(SplineSeries)
 public:
@@ -30,11 +34,15 @@ public:
     Qt::PenCapStyle capStyle() const;
     void setCapStyle(Qt::PenCapStyle newCapStyle);
 
+    bool isOptimized() const;
+    void setOptimized(bool optimized);
+
     QList<QPointF> &getControlPoints();
 
 Q_SIGNALS:
     void widthChanged();
     void capStyleChanged();
+    Q_REVISION(6, 11) void optimizedChanged(bool newOptimized);
 
 protected:
     QSplineSeries(QSplineSeriesPrivate &dd, QObject *parent = nullptr);
