@@ -1,5 +1,7 @@
 // Copyright (C) 2024 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 #include <QtCore/QAbstractItemModel>
 #include <QtGraphs/QXYModelMapper>
 #include <QtGraphs/QXYSeries>
@@ -10,6 +12,8 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_LOGGING_CATEGORY(lcModelMapper2D, "qt.graphs2d.modelmapper")
+
 Q_TRACE_PREFIX(qtgraphs,
             "QT_BEGIN_NAMESPACE" \
             "class QXYModelMapper;" \
@@ -183,7 +187,7 @@ Q_TRACE_POINT(qtgraphs, QGraphs2DXYModelMapperOnModelUpdated_exit);
 */
 
 /*!
-    \qmlsignal QXYModelMapper::seriesChanged()
+    \qmlsignal XYModelMapper::seriesChanged()
 
     This signal is emitted when the series that the mapper is connected to changes.
 */
@@ -816,10 +820,10 @@ void QXYModelMapperPrivate::initializeXYFromModel()
         int count = m_orientation == Qt::Vertical ? m_model->rowCount() : m_model->columnCount();
         if (count > 0) {
             if (!xIndex.isValid()) {
-                qWarning("%ls Invalid X coordinate index in model mapper.",
+                qCWarning(lcModelMapper2D, "%ls Invalid X coordinate index in model mapper.",
                          qUtf16Printable(QString::fromUtf8(__func__)));
             } else if (!yIndex.isValid()) {
-                qWarning("%ls Invalid Y coordinate index in model mapper.",
+                qCWarning(lcModelMapper2D, "%ls Invalid Y coordinate index in model mapper.",
                          qUtf16Printable(QString::fromUtf8(__func__)));
             }
         }
@@ -827,4 +831,7 @@ void QXYModelMapperPrivate::initializeXYFromModel()
 
     blockSeriesSignals(false);
 }
+
 QT_END_NAMESPACE
+
+#include "moc_qxymodelmapper.cpp"
