@@ -1,5 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "baritemmodelhandler_p.h"
 #include "qbar3dseries_p.h"
@@ -293,12 +295,12 @@ void BarItemModelHandler::resolveModel()
             }
         }
 
-        if (!m_proxy->series()->rowLabels().isEmpty())
-            rowLabels = m_proxy->series()->rowLabels();
+        if (!m_proxy->series()->customRowLabels().isEmpty())
+            rowLabels = m_proxy->series()->customRowLabels();
         else
             rowLabels = rowList;
-        if (!m_proxy->series()->columnLabels().isEmpty())
-            columnLabels = m_proxy->series()->columnLabels();
+        if (!m_proxy->series()->customColumnLabels().isEmpty())
+            columnLabels = m_proxy->series()->customColumnLabels();
         else
             columnLabels = columnList;
 
@@ -311,3 +313,5 @@ void BarItemModelHandler::resolveModel()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_baritemmodelhandler_p.cpp"
