@@ -7,8 +7,22 @@
 
 #include <QtGui/qquaternion.h>
 
+#include <qtgraphs_tracepoints_p.h>
+
 QT_BEGIN_NAMESPACE
 
+Q_TRACE_PREFIX(qtgraphs,
+              "QT_BEGIN_NAMESPACE" \
+              "#include <qnamespace.h>" \
+              "class QCustom3DVolume;" \
+              "QT_END_NAMESPACE"
+          )
+
+Q_TRACE_METADATA(qtgraphs, "ENUM { } Qt::Axis;")
+
+Q_TRACE_POINT(qtgraphs, QGraphs3DCustomVolumeRenderSlice_entry, Qt::Axis axis, int index);
+Q_TRACE_POINT(qtgraphs, QGraphs3DCustomVolumeRenderSlice_exit)
+
 /*!
  * \class QCustom3DVolume
  * \inmodule QtGraphs
@@ -1002,7 +1016,7 @@ void QCustom3DVolume::setAlphaMultiplier(float mult)
 {
     Q_D(QCustom3DVolume);
     if (mult >= 0.0f) {
-        if (qFuzzyCompare(d->m_alphaMultiplier, mult)) {
+        if (qFuzzyCompare(d->m_alphaMultiplier + 1, mult + 1)) {
             qCDebug(lcProperties3D, "%s value is already set to: %.1f",
                     qUtf8Printable(QLatin1String(__FUNCTION__)), mult);
             return;
@@ -1425,6 +1439,7 @@ QImage QCustom3DVolumePrivate::renderSlice(Qt::Axis axis, int index)
     if (index < 0)
         return QImage();
 
+    Q_TRACE_SCOPE(QGraphs3DCustomVolumeRenderSlice, axis, index);
     int x;
     int y;
     if (axis == Qt::XAxis) {
@@ -1516,3 +1531,5 @@ int QCustom3DVolumePrivate::multipliedAlphaValue(int alpha)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qcustom3dvolume.cpp"
