@@ -1,5 +1,7 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 
 #include "qabstract3daxis_p.h"
 #include "qscatter3dseries_p.h"
@@ -492,3 +494,5 @@ void QScatterDataProxyPrivate::setSeries(QAbstract3DSeries *series)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qscatterdataproxy.cpp"
